#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

""" AbstractShutter class - interface for shutter type devices.
Define open/close methods and is_open property.
Overload BaseValueEnum
"""

import abc
from enum import (
    Enum,
    unique,
)

from mxcubecore.HardwareObjects.abstract.AbstractNState import AbstractNState

__copyright__ = """ Copyright 2016-2023 by the MXCuBE collaboration """
__license__ = "LGPLv3+"


@unique
class BaseValueEnum(Enum):
    """Defines only the compulsory values."""

    OPEN = "OPEN"
    CLOSED = "CLOSED"
    UNKNOWN = "UNKNOWN"


class AbstractShutter(AbstractNState):
    """Abstract base class for shutter type objects."""

    __metaclass__ = abc.ABCMeta
    VALUES = BaseValueEnum

    @property
    def is_open(self):
        """Check if the shutter is open.
        Returns:
            (bool): True if open, False otherwise.
        """
        return self.get_value() == self.VALUES.OPEN

    def open(self, timeout=None):
        """Open the shutter.
        Args:
            timeout(float): optional - timeout [s],
                            If timeout == 0: return at once and do not wait
                            if timeout is None: wait forever.
        """
        self.set_value(self.VALUES.OPEN, timeout=timeout)

    def close(self, timeout=None):
        """Close the shutter.
        Args:
            timeout(float): optional - timeout [s],
                            If timeout == 0: return at once and do not wait
                            if timeout is None: wait forever.
        """
        self.set_value(self.VALUES.CLOSED, timeout=timeout)



from bliss.config import channels
from TangoKeithleyPhotonFlux import TangoKeithleyPhotonFlux


class ID232PhotonFlux(TangoKeithleyPhotonFlux):
    def __init__(self, *args, **kwargs):
        TangoKeithleyPhotonFlux.__init__(self, *args, **kwargs)

    def init(self):
        self._flux_chan = channels.Channel("mxcube:flux")
        TangoKeithleyPhotonFlux.init(self)

    def emitValueChanged(self, flux=None):
        self._flux_chan.value = flux
        return TangoKeithleyPhotonFlux.emitValueChanged(self, flux)



import datetime
import json
import logging
import subprocess
import xmlrpc.client

import gevent
from devtools import debug
from pydantic.v1 import (
    BaseModel,
    Field,
)
from typing_extensions import (
    Literal,
    Optional,
)

from mxcubecore import HardwareRepository as HWR
from mxcubecore.model.common import (
    BeamlineParameters,
    CommonCollectionParamters,
    ISPYBCollectionParameters,
    LegacyParameters,
    PathParameters,
    StandardCollectionParameters,
)
from mxcubecore.queue_entry.base_queue_entry import BaseQueueEntry

DEFAULT_MAX_FREQ = 925


class SSXPathParameters(PathParameters):
    use_experiment_name: bool = Field(
        True, description="Whether to use the experiment name in the data path"
    )


class BaseUserCollectionParameters(BaseModel):
    exp_time: float = Field(95e-6, gt=0, lt=1, description="s")
    sub_sampling: Literal[1, 2, 4, 6, 8] = Field(1)
    take_pedestal: bool = Field(True)
    reject_empty_frames: bool = Field(True)

    frequency: float = Field(
        float(HWR.beamline.diffractometer.get_property("max_freq", DEFAULT_MAX_FREQ)),
        description="Hz",
    )


class SsxBaseQueueTaskParameters(BaseModel):
    path_parameters: SSXPathParameters
    common_parameters: CommonCollectionParamters
    collection_parameters: StandardCollectionParameters
    legacy_parameters: LegacyParameters
    lims_parameters: Optional[ISPYBCollectionParameters]

    def update_dependent_fields(field_data):
        return {}

    @staticmethod
    def ui_schema():
        return json.dumps(
            {
                "ui:order": [
                    "num_images",
                    "exp_time",
                    "osc_range",
                    "osc_start",
                    "resolution",
                    "transmission",
                    "energy",
                    "vertical_spacing",
                    "horizontal_spacing",
                    "nb_lines",
                    "nb_samples_per_line",
                    "motor_top_left_x",
                    "motor_top_left_y",
                    "motor_top_left_z",
                    "motor_top_right_x",
                    "motor_top_right_y",
                    "motor_top_right_z",
                    "motor_bottom_left_x",
                    "motor_bottom_left_y",
                    "motor_bottom_left_z",
                    "chip_type",
                    "take_pedestal",
                    "sub_sampling",
                    "*",
                ],
                "ui:submitButtonOptions": {
                    "norender": "true",
                },
                "sub_sampling": {"ui:readonly": "true"},
                "frequency": {"ui:readonly": "true"},
            }
        )


class SsxBaseQueueEntry(BaseQueueEntry):
    """
    Defines common SSX collection methods.
    """

    def __init__(self, view, data_model):
        super().__init__(view=view, data_model=data_model)
        self._beamline_values = None
        self._use_nicoproc = False
        self._use_besproc = False
        self._processing_host = "http://lid29control-2:9998"
        self._current_data_path = None

        self.__pedestal_task = None
        self.__stop_req = False

    def get_data_path(self):
        return self._current_data_path

    def take_pedestal(self):
        params = self._data_model._task_data.user_collection_parameters

        exp_time = self._data_model._task_data.user_collection_parameters.exp_time
        freq = self._data_model._task_data.user_collection_parameters.frequency
        sub_sampling = (
            self._data_model._task_data.user_collection_parameters.sub_sampling
        )

        effect_freq = freq / sub_sampling
        logging.getLogger("user_level_log").info(
            f"Frequency for pedestal: {freq} / {sub_sampling} = {effect_freq}"
        )

        data_root_path = self.get_data_path()

        packet_fifo_depth = 20000

        save_raw = False

        if params.take_pedestal:
            HWR.beamline.control.safshut_oh2.close()
            if not hasattr(HWR.beamline.control, "lima2_jungfrau_pedestal_scans"):
                HWR.beamline.control.load_script("id29_lima2.py")

            pedestal_dir = HWR.beamline.detector.find_next_pedestal_dir(
                data_root_path, "pedestal"
            )
            sls_detectors = "/users/blissadm/local/sls_detectors"
            lima2_path = f"{sls_detectors}/lima2"
            cl_source_path = f"{lima2_path}/processings/common/fai/kernels"

            disable_saving_list = []
            if not save_raw:
                disable_saving_list.append("raw")
            disable_saving = ",".join(disable_saving_list)

            saving_compression = dict(
                raw="zip",
                average="zip",
            )

            logging.getLogger("user_level_log").info(
                f"Storing pedestal in {pedestal_dir}"
            )
            subprocess.Popen(
                "mkdir --parents %s && chmod -R 755 %s" % (pedestal_dir, pedestal_dir),
                shell=True,
                stdin=None,
                stdout=None,
                stderr=None,
                close_fds=True,
            ).wait()

            if len(HWR.beamline.detector.lima2_device.recvs) == 4:
                rr = "rr4"
            else:
                rr = "rr"
            det_name = f"lima2_jungfrau4m_{rr}_smx"
            detector = getattr(HWR.beamline.control, det_name)

            def pedestal_func():
                HWR.beamline.control.lima2_jungfrau_pedestal_scans(
                    detector,
                    exp_time,
                    effect_freq,
                    1000,
                    pedestal_dir,
                    "pedestal.h5",
                    disable_saving=disable_saving,
                    print_params=True,
                    det_params={"packet_fifo_depth": packet_fifo_depth},
                    cl_source_path=cl_source_path,
                    saving_compression=saving_compression,
                )

            self.__stop_req = False

            self.__pedestal_task = gevent.spawn(pedestal_func)
            try:
                while not self.__pedestal_task.ready():
                    if self.__stop_req:
                        self.__pedestal_task.kill()
                        return
                    gevent.sleep(0.1)
                self.__pedestal_task.join()
            finally:
                self.__pedestal_task = None

            subprocess.Popen(
                "cd %s && rm -f pedestal.h5 && ln -s %s/pedestal.h5"
                % (data_root_path, pedestal_dir),
                shell=True,
                stdin=None,
                stdout=None,
                stderr=None,
                close_fds=True,
            ).wait()

        HWR.beamline.control.mtdsx.wait_move()

    def start_processing(self, exp_type):
        data_root_path = self.get_data_path()

        if self._use_nicoproc:
            logging.getLogger("user_level_log").info(f"Starting NICOPROC")
            self._start_nico_processing(
                self._beamline_values,
                self._data_model._task_data,
                data_root_path,
                experiment_type=exp_type,
            )
        else:
            logging.getLogger("user_level_log").info(f"Not using NICOPROC")

        if self._use_besproc:
            logging.getLogger("user_level_log").info(f"Using BES PROC")
            HWR.beamline.workflow.start(
                ["modelpath", "SSX", "data_path", data_root_path]
            )
        else:
            logging.getLogger("user_level_log").info(f"BES PROC False")

    def prepare_acquisition(self):
        exp_time = self._data_model._task_data.user_collection_parameters.exp_time
        fname_prefix = self._data_model._task_data.path_parameters.prefix
        num_images = self._data_model._task_data.user_collection_parameters.num_images

        data_root_path = self.get_data_path()
        reject_empty_frames = (
            self._data_model._task_data.user_collection_parameters.reject_empty_frames
        )

        logging.getLogger("user_level_log").info(f"Preparing detector")
        HWR.beamline.detector.wait_ready()
        HWR.beamline.detector.stop_acquisition()
        HWR.beamline.detector.prepare_acquisition(
            num_images,
            exp_time,
            data_root_path,
            fname_prefix,
            dense_skip_nohits=reject_empty_frames,
        )
        HWR.beamline.detector.wait_ready()
        logging.getLogger("user_level_log").info(f"Detector prepared, continuing !")

    def _monitor_collect(self):
        for i in range(1, 99):
            self.emit_progress(i / 100.0)
            gevent.sleep(0.1)

    def execute(self):
        super().execute()
        debug(self._data_model._task_data)

        self._monitor_task = gevent.spawn(self._monitor_collect)

    def pre_execute(self):
        super().pre_execute()
        self._current_data_path = self.get_data_model().get_path_template().directory

        logging.getLogger("user_level_log").info(f"Moving detector table")
        HWR.beamline.control.mtdsx.wait_move()
        HWR.beamline.control.mtdsx.move(0, wait=False)
        self._beamline_values = self.get_current_beamline_values()
        self._data_model._task_data.lims_parameters = self.get_additional_lims_values()
        self.emit_progress(0)

    def post_execute(self):
        super().post_execute()
        data_root_path = self.get_data_path()
        self._data_model._task_data.lims_parameters.end_time = datetime.datetime.now()

        parameters = {}
        parameters["collection_parameters"] = self._data_model._task_data
        parameters["data_path"] = data_root_path
        parameters["beamline_parameters"] = self._beamline_values
        parameters["extra_lims_values"] = self._data_model._task_data.lims_parameters

        HWR.beamline.lims.finalize_data_collection(parameters)

        logging.getLogger("user_level_log").info(f"Moving detector back")
        HWR.beamline.control.mtdsx.move(1000, wait=False)

        if HWR.beamline.control.safshut_oh2.state.name == "OPEN":
            logging.getLogger("user_level_log").info(f"Closing OH2 safety shutter")
            HWR.beamline.control.safshut_oh2.close()

        # self.emit_progress(1)

    def emit_progress(self, progress):
        HWR.beamline.collect.emit_progress(progress)

    def stop(self):
        self.__stop_req = True

        super().stop()

        if self.__pedestal_task is not None:
            return

        if HWR.beamline.control.safshut_oh2.state.name == "OPEN":
            HWR.beamline.control.safshut_oh2.close()
            logging.getLogger("user_level_log").info("shutter closed")

        HWR.beamline.detector.stop_acquisition()

    def _start_processing(self, dc_parameters, file_paramters):
        param = {
            "exposure": dc_parameters["oscillation_sequence"][0]["exposure_time"],
            "detector_distance": dc_parameters["detectorDistance"],
            "wavelength": dc_parameters["wavelength"],
            "orgx": dc_parameters["xBeam"],
            "orgy": dc_parameters["yBeam"],
            "oscillation_range": dc_parameters["oscillation_sequence"][0]["range"],
            "start_angle": dc_parameters["oscillation_sequence"][0]["start"],
            "number_images": dc_parameters["oscillation_sequence"][0][
                "number_of_images"
            ],
            "image_first": dc_parameters["oscillation_sequence"][0][
                "start_image_number"
            ],
            "fileinfo": file_paramters,
        }

        logging.getLogger("HWR").info("NICOPROC START")

        with xmlrpc.client.ServerProxy(self._processing_host) as p:
            p.start(param)

    def _start_nico_processing(self, beamline_values, params, path, experiment_type=""):
        param = {
            "exposure": params.user_collection_parameters.exp_time,
            "detector_distance": beamline_values.detector_distance,
            "wavelength": beamline_values.wavelength,
            "orgx": beamline_values.beam_x,
            "orgy": beamline_values.beam_y,
            "oscillation_range": params.collection_parameters.osc_range,
            "start_angle": params.collection_parameters.osc_start,
            "number_images": params.user_collection_parameters.num_images,
            "image_first": params.collection_parameters.first_image,
            "fileinfo": params.path_parameters.dict(),
            "root_path": path,
            "experiment_type": experiment_type,
        }

        logging.getLogger("HWR").info("NICOPROC START")

        try:
            with xmlrpc.client.ServerProxy(self._processing_host) as p:
                p.start(param)
        except Exception:
            logging.getLogger("HWR").exception("")

    def _stop_processing(self):
        logging.getLogger("HWR").info("NICOPROC STOP")

        with xmlrpc.client.ServerProxy(self._processing_host) as p:
            p.stop()

    def get_current_beamline_values(self):
        return BeamlineParameters(
            **{
                "energy": HWR.beamline.energy.get_value(),
                "wavelength": HWR.beamline.energy.get_wavelength(),
                "resolution": HWR.beamline.resolution.get_value(),
                "transmission": HWR.beamline.transmission.get_value(),
                "detector_distance": 103.0,
                "beam_x": HWR.beamline.detector.get_beam_position()[0],
                "beam_y": HWR.beamline.detector.get_beam_position()[1],
                "beam_size_x": 4,
                "beam_size_y": 2,
                "beam_shape": "gaussian",
                "energy_bandwidth": 1,
            }
        )

    def get_additional_lims_values(self):
        return ISPYBCollectionParameters(
            **{
                "flux_start": 3e15,
                "flux_end": 3e15,
                "start_time": datetime.datetime.now(),
                "end_time": datetime.datetime.now(),
                "chip_model": "",
                "polarisation": 0.99,
                "mono_stripe": "PdB4C",
            }
        )



#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

"""
[Name]
Qt4_LimaVideo

[Description]
HwObj used to grab images via LImA library or Lima Tango server.

[Configuration]
See example below.  To select between "Library" or "Tango" simply
use and configure the field <address> (for Library)
or <tangoname> (for Tango)
in the XML file.


Example Hardware Object XML file :
==================================
<object class="Qt4_LimaVideo">
   <type>basler</type>
   <encoding>yuv422p</encoding>
   <address>84.89.227.6</address>
   <gain>0.5</gain>
   <exposure>0.01</exposure>
   <mirror>(False, False)</mirror>
   <interval>30</interval>
</object>
"""
import logging
import os
import struct
import time

from mxcubecore.HardwareObjects.QtLimaVideo import QtLimaVideo

try:
    from Lima import Core
except Exception:
    pass

try:
    from Lima import Prosilica
except ImportError as e:
    pass

try:
    from Lima import Basler
except ImportError as e:
    pass


class PX2Qt4_LimaVideo(QtLimaVideo):
    default_exposure_time = 0.5

    def init(self):
        exposure_time = self.get_property("exposure_time")

        if exposure_time is not None:
            self.exposure_time = float(exposure_time)
        else:
            self.exposure_time = self.default_exposure_time

        QtLimaVideo.init(self)

    def set_cam_encoding(self, set_cam_encoding):
        if cam_encoding == "yuv422p":
            self.video.setMode(Core.YUV422)
            self.decoder = self.yuv_2_rgb
        elif cam_encoding == "y8":
            self.video.setMode(Core.Y8)
            self.decoder = self.y8_2_rgb
        elif cam_encoding == "y16":
            self.video.setMode(Core.Y16)

    def get_image(self):
        image = self.video.getLastImage()
        raw_buffer = image.buffer()
        return raw_buffer, image.width(), image.height()

    def set_exposure_time(self, exposure_time):
        was_live = False
        print("Setting exposure to %s" % exposure_time)
        if self.get_video_live():
            was_live = True
            self.set_video_live(False)
            print("Stopped")

        self.video.setExposure(exposure_time)

        if was_live:
            print("Starting")
            self.set_video_live(True)


def test_hwo():
    import time

    from mxcubecore import HardwareRepository as HWR
    from mxcubecore.utils.qt_import import *

    hwr = HWR.get_hardware_repository()
    hwr.connect()

    hwo = hwr.get_hardware_object("/singleton_objects/limavideo")

    print("Image dimensions: ", hwo.get_image_dimensions())
    print("Live Mode: ", hwo.get_video_live())

    app = QApplication([])

    win = QMainWindow()
    lab = QLabel("toto")

    print("Image dimensions: ", hwo.get_image_dimensions())
    hwo.set_video_live(True)
    hwo.set_exposure_time(0.05)
    time.sleep(1)

    qimg = hwo.get_new_image()
    px = QPixmap(qimg)

    px = px.scaled(QSize(px.width() * 0.5, px.height() * 0.5))

    lab.setPixmap(px)
    win.setCentralWidget(lab)
    win.show()
    app.exec_()


if __name__ == "__main__":
    test_hwo()



"""
EPICS implementation of AbstractNState.
Example xml file:
<object class="LNLS.LNLSInOut">
  <channel type="epics" name="epicsActuator_val">SOL:S:m4.SET</channel>
  <channel type="epics" name="epicsActuator_rbv" polling="500">SOL:S:m4.SET</channel>
  <username>Microdiff backlight</username>
  <motor_name>BackLightIsOn</motor_name>
  <values>{"in": True, "out": False}</values>
</object>
"""
import logging
from enum import Enum

from mxcubecore.HardwareObjects.abstract.AbstractNState import AbstractNState
from mxcubecore.HardwareObjects.LNLS.EPICSActuator import EPICSActuator


class EPICSNState(AbstractNState, EPICSActuator):

    def __init__(self, name):
        AbstractNState.__init__(self, name)
        self.username = "unknown"
        self.actuatorState = "unknown"
        self.state_attr = None

    def init(self):
        AbstractNState.initialise_values(self)
        self.username = self.getProperty("username")
        self.states = dict((item.value, item.name) for item in self.VALUES)
        self.moves = dict((item.name, item.value) for item in self.VALUES)
        self.get_actuator_state()

    def connectNotify(self, signal):
        if signal == "actuatorStateChanged":
            self.valueChanged(self.state_attr)

    def _set_value(self, value):
        """Set device to value
        Args:
            value (str, int, float or enum): Value to be set.
        """
        if isinstance(value, Enum):
            try:
                value = value.value[0]
            except TypeError:
                value = value.value
        EPICSActuator._set_value(self, value)
        #self.set_channel_value(self.ACTUATOR_VAL, value)
        self.update_state()

    def get_value(self):
        """Get the device value
        Returns:
            (Enum): Enum member, corresponding to the value or UNKNOWN.
        """
        value = EPICSActuator.get_value(self)
        #value = self.get_channel_value(self.ACTUATOR_RBV)
        return self.value_to_enum(value)

    def get_actuator_state(self, read=False):
        current_value = self.get_value()
        self.state_attr = current_value.value # Bool
        self.actuatorState = self.states.get(self.state_attr, "unknown") # Name
        return self.actuatorState

    def valueChanged(self, value):
        enum_val = self.value_to_enum(value)
        self.set_value(enum_val)
        self.actuatorState = self.states.get(value, "unknown")
        self.emit("actuatorStateChanged", (self.actuatorState,))
    
    def actuatorIn(self, wait=True, timeout=None):
        try:
            self.state_attr = self.moves["in"]
            self.valueChanged(self.state_attr)
        except BaseException:
            logging.getLogger("user_level_log").error(
                "Cannot put %s in", self.username
            )

    def actuatorOut(self, wait=True, timeout=None):
        try:
            self.state_attr = self.moves["out"]
            self.valueChanged(self.state_attr)
        except BaseException as e:
            logging.getLogger("user_level_log").error(
                "Cannot put %s out: %s", (self.username, e)
            )


from mxcubecore.BaseHardwareObjects import HardwareObject

from .BeamCmds import ControllerCommand


class ID30A3BeamCmds(HardwareObject):
    def __init__(self, *args):
        HardwareObject.__init__(self, *args)

    def init(self):
        controller = self.get_object_by_role("controller")
        controller.detcover.set_in()
        self.centrebeam = ControllerCommand("Centre beam", controller.centrebeam)
        self.quick_realign = ControllerCommand(
            "Quick realign", controller.quick_realign
        )
        self.anneal = ControllerCommand("Anneal", controller.anneal_procedure)
        self.jetvideo = ControllerCommand("Jet video", controller.jet_video)

    def get_commands(self):
        return [self.centrebeam, self.quick_realign, self.anneal, self.jetvideo]



# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

import logging
import weakref

import qt

try:
    import Queue as queue
except ImportError:
    import queue

from mxcubecore.CommandContainer import (
    ChannelObject,
    CommandObject,
    ConnectionError,
)

try:
    import PyTango
except ImportError:
    logging.getLogger("HWR").warning("Tango support is not available.")


__copyright__ = """ Copyright © 2010 - 2020 by MXCuBE Collaboration """
__license__ = "LGPLv3+"


class BoundMethodWeakref:
    def __init__(self, bound_method):
        self.func_ref = weakref.ref(bound_method.im_func)
        self.obj_ref = weakref.ref(bound_method.im_self)

    def __call__(self):
        obj = self.obj_ref()
        if obj is not None:
            func = self.func_ref()
            if func is not None:
                return func.__get__(obj)

    def __hash__(self):
        return id(self)

    def __cmp__(self, other):
        # NBNB 'cmp' and __cmp__ is Python 2 ony
        if other.__class__ == self.__class__:
            return cmp((self.func_ref, self.obj_ref), (other.func_ref, other.obj_ref))
        else:
            return cmp(self, other)


class PoolCommand(CommandObject):
    def __init__(self, name, macro_name, tango_name=None, username=None, **kwargs):
        CommandObject.__init__(self, name, username, **kwargs)

        self.command = "RunMacro"
        self.device_name = tango_name
        self.macro_name = macro_name

        try:
            self.device = PyTango.DeviceProxy(self.device_name)
        except PyTango.DevFailed as traceback:
            last_error = traceback[-1]
            logging.getLogger("HWR").error(
                "%s: %s", str(self.name()), last_error["desc"]
            )
            self.device = None
        else:
            try:
                self.device.ping()
            except PyTango.ConnectionFailed:
                self.device = None
                raise ConnectionError

    def __call__(self, *args, **kwargs):
        self.emit("commandBeginWaitReply", (str(self.name()),))

        if self.device is not None:
            try:
                tango_cmd_object = getattr(self.device, self.command)
                args = (self.macro_name,) + args
                logging.getLogger("HWR").debug(
                    "%s: %s, args=%s", str(self.name()), tango_cmd_object, args
                )
                ret = tango_cmd_object(
                    args
                )  # eval('self.device.%s(*%s)' % (self.command, args))
            except PyTango.DevFailed as error_dict:
                logging.getLogger("HWR").error(
                    "%s: Tango, %s", str(self.name()), error_dict
                )
            except Exception:
                logging.getLogger("HWR").exception(
                    "%s: an error occured when calling Tango command %s",
                    str(self.name()),
                    self.command,
                )
            else:
                self.emit("commandReplyArrived", (ret, str(self.name())))
                return ret

        self.emit("commandFailed", (-1, self.name()))

    def abort(self):
        self.device.abort()
        self.emit("commandAborted", (str(self.name()),))

    def is_connected(self):
        return self.device is not None


def process_tango_events():
    while not PoolChannel._tangoEventsQueue.empty():
        try:
            event = PoolChannel._tangoEventsQueue.get_nowait()
        except queue.Empty:
            break
        else:
            try:
                receiver_cb_ref = PoolChannel._eventReceivers[id(event)]
                receiver_cb = receiver_cb_ref()
                if receiver_cb is not None:
                    try:
                        receiver_cb(event.attr_value.value)
                    except AttributeError:
                        pass
            except KeyError:
                pass


class PoolChannel(ChannelObject):
    _tangoEventsQueue = queue.Queue()
    _eventReceivers = {}
    _tangoEventsProcessingTimer = qt.QTimer()

    # start Tango events processing timer
    qt.QObject.connect(
        _tangoEventsProcessingTimer, qt.SIGNAL("timeout()"), process_tango_events
    )
    _tangoEventsProcessingTimer.start(20)

    def __init__(
        self,
        name,
        attribute_name,
        tango_name=None,
        username=None,
        polling=None,
        timeout=10000,
        **kwargs
    ):
        ChannelObject.__init__(self, name, username, **kwargs)

        self.attribute_name = attribute_name
        self.device_name = tango_name
        self.device = None
        self.value = None
        self.polling = polling
        self.__connections = 0
        self.__value = None
        self.polling_timer = None
        self.timeout = int(timeout)

        logging.getLogger("HWR").debug(
            "creating Tango attribute %s/%s, polling=%s, timeout=%d",
            self.device_name,
            self.attribute_name,
            polling,
            self.timeout,
        )

        try:
            self.device = PyTango.DeviceProxy(self.device_name)
        except PyTango.DevFailed as traceback:
            last_error = traceback[-1]
            logging.getLogger("HWR").error(
                "%s: %s", str(self.name()), last_error["desc"]
            )
        else:
            try:
                self.device.ping()
            except PyTango.ConnectionFailed:
                self.device = None
                raise ConnectionError
            else:
                self.device.set_timeout_millis(self.timeout)

                if isinstance(polling, int):
                    self.polling_timer = qt.QTimer()
                    self.polling_timer.connect(
                        self.polling_timer, qt.SIGNAL("timeout()"), self.poll
                    )
                    self.polling_timer.start(polling)
                else:
                    if polling == "events":
                        # try to register event
                        try:
                            self.device.subscribe_event(
                                self.attribute_name, PyTango.EventType.CHANGE, self, []
                            )
                        except PyTango.EventSystemFailed:
                            pass

    def push_event(self, event):
        PoolChannel._eventReceivers[id(event)] = BoundMethodWeakref(self.update)
        PoolChannel._tangoEventsQueue.put(event)

    def poll(self):
        try:
            value = self.device.read_attribute(self.attribute_name).value
        except Exception:
            logging.getLogger("HWR").exception(
                "%s: could not poll attribute %s", str(self.name()), self.attribute_name
            )

            self.polling_timer.stop()
            if not hasattr(self, "_statepolling_timer"):
                self._statepolling_timer = qt.QTimer()
                self._statepolling_timer.connect(
                    self._statepolling_timer, qt.SIGNAL("timeout()"), self.state_polling
                )
            self.device.set_timeout_millis(50)
            self._statepolling_timer.start(5000)
            value = None
            self.emit("update", (None,))
        else:
            if value != self.value:
                self.update(value)

    def state_polling(self):
        """Called when polling has failed"""
        try:
            s = self.device.State()
        except Exception:
            pass
            # logging.getLogger("HWR").exception("Could not read State attribute")
        else:
            if s == PyTango.DevState.OFF:
                return

            self._statepolling_timer.stop()
            self.device.set_timeout_millis(self.timeout)
            logging.getLogger("HWR").info(
                "%s: restarting polling on attribute %s",
                self.name(),
                self.attribute_name,
            )
            self.polling_timer.start(self.polling)

    def update(self, value=None):
        if value is None:
            value = self.get_value()

        self.value = value
        self.emit("update", value)

    def get_value(self):
        self.value = self.device.read_attribute(self.attribute_name).value

        return self.value

    def set_value(self, new_value):
        # newval = PyTango.AttributeValue()
        # newval.value = newValue
        # self.device.write_attribute(self.attribute_name, newval)
        attr = PyTango.AttributeProxy(self.device_name + "/" + self.attribute_name)
        a = attr.read()
        a.value = new_value
        attr.write(a)

    def is_connected(self):
        return self.device is not None



# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

__copyright__ = """Copyright The MXCuBE Collaboration"""
__license__ = "LGPLv3+"

import logging
import sys
import urllib.request

from mxcubecore.BaseHardwareObjects import HardwareObject

__credits__ = ["MXCuBE collaboration"]


class P11DoorInterlock(HardwareObject):
    def __init__(self, name):
        HardwareObject.__init__(self, name)

        self.door_interlock_state = None
        self.simulationMode = False

    def init(self):
        self.door_interlock_state = self.STATES.READY

    def connected(self):
        self.set_is_ready(True)

    def disconnected(self):
        self.set_is_ready(False)

    def door_is_interlocked(self):
        return self.door_interlock_state in [self.STATES.READY]

    def get_state(self):
        return self.door_interlock_state

    def unlock(self):
        self.breakInterlockEH()

    def unlock_door_interlock(self):
        if self.door_interlock_state == "locked_active":
            self.door_interlock_state = "unlocked"
        self.emit("doorInterlockStateChanged", self.door_interlock_state, "")

    def breakInterlockEH(self):
        """Command to break the interlock by sending a request to a URL."""
        if not self.simulationMode:
            logging.info("Attempting to break interlock by opening EH")
            url = self.get_property("unlockEH_url")
            success = self.fetch_url(url)
            if success:
                logging.info("EH opened successfully")
                # Update the state to reflect that the interlock was broken
                self.emit("doorInterlockStateChanged", self.door_interlock_state, "")
            else:
                logging.error("Failed to open EH")

    def fetch_url(self, url, timeout=3, retries=10):
        """Fetch URL with retry mechanism."""
        for attempt in range(retries):
            try:
                result = urllib.request.urlopen(url, None, timeout).readlines()
                logging.info(f"Successfully fetched URL: {url}")
                return True  # Success
            except Exception as e:
                logging.error(f"Error fetching URL: {url} on attempt {attempt + 1}")
                logging.error(f"Error details: {sys.exc_info()}")
                if attempt + 1 == retries:
                    logging.error(f"All {retries} attempts failed.")
                    return False  # Failed after retries



#! /usr/bin/env python
# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

"""Beamline class serving as singleton container for links to top-level HardwareObjects

All HardwareObjects
"""

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

__copyright__ = """ Copyright © 2019 by the MXCuBE collaboration """
__license__ = "LGPLv3+"
__author__ = "Rasmus H Fogh"

from collections import OrderedDict

from mxcubecore.HardwareObjects.Beamline import Beamline


class EMBLBeamline(Beamline):
    """Beamline class serving as singleton container for links to HardwareObjects"""

    # Roles of defined objects and the category they belong to
    # NB the double underscore is deliberate - attribute must be hidden from subclasses
    __content_roles = []

    def __init__(self, name):
        """

        Args:
            name (str) : Object name, generally saet to teh role name of the object
        """
        super(EMBLBeamline, self).__init__(name)

    # NB this function must be re-implemented in nested subclasses
    @property
    def all_roles(self):
        """Tuple of all content object roles, indefinition and loading order

        Returns:
            tuple[text_str, ...]
        """
        return super(EMBLBeamline, self).all_roles + tuple(self.__content_roles)

    # Additional properties

    @property
    def ppu_control(self):
        """PPU control Hardware object

        Returns:
            Optional[HardwareObject]:
        """
        return self._objects.get("ppu_control")

    __content_roles.append("ppu_control")


    @property
    def front_light(self):
        """Diffractometer front light

        Returns:
            Optional[HardwareObject]:
        """
        return self._objects.get("front_light")

    __content_roles.append("front_light")


    @property
    def back_light(self):
        """Diffractometer back light

        Returns:
            Optional[HardwareObject]:
        """
        return self._objects.get("back_light")

    __content_roles.append("back_light")

    @property
    def beam_centering(self):
        return self._objects.get("beam_centering")

    __content_roles.append("beam_centering")


    # Additional procedures

    # NB this is just an example of a beamline-specific procedure description
    @property
    def xray_centring(self):
        """ X-ray Centring Procedure

        NB EMBLXrayCentring is defined in EMBL-specific code, like EMBLBeamline

        Returns:
            Optional[EMBLXrayCentring]
        """
        return self._objects.get("xray_centring")

    __content_roles.append("xray_centring")
    # Registers this object as a procedure:
    Beamline._procedure_names.add("xray_centring")



import logging
import os
import sys
import time

from xaloc import XalocJob
from XSDataMXCuBEv1_3 import XSDataResultMXCuBE

sys.path.append("/beamlines/bl13/controls/devel/pycharm/ALBAClusterClient")


root = os.environ["POST_PROCESSING_SCRIPTS_ROOT"]


class ALBAClusterJob(object):
    def __init__(self, *args):
        self.job = None

    def run(self, *args):
        pass

    def wait_done(self, wait=True):

        if not self.job:
            return

        time.sleep(0.5)

        state = self.job.state

        if not wait:
            return state

        while state in ["RUNNING", "PENDING"]:
            logging.getLogger("HWR").debug("Job / is %s" % state)
            time.sleep(0.5)
            state = self.job.state

        logging.getLogger("HWR").debug(' job finished with state: "%s"' % state)
        return state

    def get_result(self, state):
        pass


class ALBAAutoprocJob(ALBAClusterJob):
    sls_script = os.path.join(root, "edna-mx/autoproc/edna-mx.autoproc.sl")

    def run(self, *args):

        jobname = os.path.basename(os.path.dirname(edna_directory))
        self.job = XalocJob(
            "edna-autoproc", jobname, self.sls_script, input_file, edna_directory
        )
        self.job.submit()


class ALBAEdnaProcJob(ALBAClusterJob):
    sls_script = os.path.join(root, "edna-mx/ednaproc/edna-mx.ednaproc.sl")

    def run(self, *args):
        collect_id, input_file, output_dir = args
        self.job = XalocJob(
            "edna-ednaproc", str(collect_id), self.sls_script, input_file, output_dir
        )
        self.job.submit()


class ALBAStrategyJob(ALBAClusterJob):

    sls_script = os.path.join(root, "edna-mx/strategy/edna-mx.strategy.sl")

    def run(self, *args):

        logging.getLogger("HWR").debug("Starting StrategyJob - ")

        input_file, results_file, edna_directory = args

        jobname = os.path.basename(os.path.dirname(edna_directory))

        self.job = XalocJob(
            "edna-strategy", jobname, self.sls_script, input_file, edna_directory
        )
        self.job.submit()

        logging.getLogger("HWR").debug("         StrategyJob - %s" % str(self.job))

        self.edna_directory = os.path.dirname(input_file)
        self.results_file = results_file

        logging.getLogger("HWR").debug("  input file: %s" % input_file)
        logging.getLogger("HWR").debug("  edna directory: %s" % self.edna_directory)

    def get_result(self, state):
        if state == "COMPLETED":
            outfile = os.path.join(
                self.edna_directory, "ControlInterfaceToMXCuBEv1_3_dataOutput.xml"
            )

            logging.getLogger("HWR").debug("Job / state is COMPLETED")
            logging.getLogger("HWR").debug("  looking for file: %s" % outfile)
            if os.path.exists(outfile):
                job_output = open(outfile).read()
                open(self.results_file, "w").write(job_output)
                result = XSDataResultMXCuBE.parseFile(self.results_file)
            else:
                logging.getLogger("HWR").debug(
                    "EDNA Job finished without success / cannot find output file "
                )
                result = ""
        else:
            logging.getLogger("HWR").debug(
                "EDNA Job finished without success / state was %s" % (job.state)
            )
            result = ""

        return result



# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

__copyright__ = """Copyright The MXCuBE Collaboration"""
__license__ = "LGPLv3+"

import time
import urllib
from enum import (
    Enum,
    unique,
)

import gevent

import mxcubecore.HardwareObjects.abstract.AbstractShutter as absshut
from mxcubecore.BaseHardwareObjects import HardwareObjectState
from mxcubecore.HardwareObjects.abstract.AbstractShutter import AbstractShutter

__credits__ = ["DESY P11"]
__license__ = "LGPLv3+"
__category__ = "General"


class P11Shutter(AbstractShutter):
    """
    P11 Shutter define interface to Tango shutters at DESY P11
    """

    default_timeout = 6

    @unique
    class BaseValueEnum(Enum):
        """Defines only the compulsory values."""

        OPEN = "OPEN"
        CLOSED = "CLOSED"
        MOVING = "MOVING"
        UNKNOWN = "UNKNOWN"

    VALUES = BaseValueEnum

    def __init__(self, name):

        super().__init__(name)

        self.simulation = False
        self.simulated_opened = True
        self.simulated_moving = False

        self.url_open = None
        self.url_closee = None
        self.cmd_started = 0

        self.chan_state = None

    def init(self):
        """Initilise the predefined values"""

        # if simulation is set - open and close will be mere software flags

        self.simulation = self.get_property("simulation")
        self._initialise_values()

        if not self.simulation:
            url_base = self.get_property("base_url")
            dev_open = self.get_property("device_open")
            dev_close = self.get_property("device_close")
            self.url_open = "{}&deviceName={}".format(url_base, dev_open)
            self.url_close = "{}&deviceName={}".format(url_base, dev_close)

            self.chan_state = self.get_channel_object("chanState")

            if self.chan_state is not None:
                self.chan_state.connect_signal("update", self.update_shutter_state)

            self.update_shutter_state()
        else:
            self.simulated_update()

        self.update_state(self.STATES.READY)

        super().init()

    def _initialise_values(self):
        """Add additional, known in advance states to VALUES"""
        values_dict = {item.name: item.value for item in self.VALUES}
        values_dict.update({"MOVING": "MOVING"})
        self.VALUES = Enum("ValueEnum", values_dict)

    def get_value(self):
        if self.simulation:
            return self.simulated_update()

        return self.update_shutter_state()

    def _set_value(self, value):
        if value == self.VALUES.OPEN:
            open_it = 1
        elif value == self.VALUES.CLOSED:
            open_it = 0
        else:
            self.log.debug(" ###  setting wrong value for shutter %s" % str(value))
            return

        if self.simulation:
            self.simulated_opened = open_it
            self.simulated_moving = True
            self.t1 = gevent.spawn(self.simul_do)
            self.t1.link(self.do_finish)
            self.t1.link_exception(self.do_finish_exc)
        else:
            if open_it:
                self.do_open()
            else:
                self.do_close()
            self.cmd_started = time.time()
        self.log.debug(" ###  setting value value for shutter done")

    def do_open(self, timeout=3):
        self.log.debug(" OPENING SHUTTER (web request)")
        result = urllib.request.urlopen(self.url_open, None, timeout).readlines()
        self.log.debug(" OPENING SHUTTER (web request) retured")

    def do_close(self, timeout=3):
        self.log.debug(" CLOSING SHUTTER (web request)")
        result = urllib.request.urlopen(self.url_close, None, timeout).readlines()
        self.log.debug(" CLOSING SHUTTER (web request) retured")

    def simul_do(self):
        self.log.debug("### starting simulated shutter move")
        gevent.sleep(1)
        self.simulated_moving = False
        self.log.debug("### updating simulated shutter")
        self.simulated_update()
        self.log.debug("### ending simulated shutter move")

    def do_finish(self, t=None):
        self.log.debug("### simulated finished")

    def do_finish_exc(self, exc=None):
        self.log.debug("### simulated finished with exception")

    def update_shutter_state(self, state=None):
        """Updates shutter state 

        :return: shutter state as str
        """
        if state is None:
            state = self.chan_state.get_value()

        self.log.debug(" SHUTTER state changed")

        if state[0] == 3:
            self.log.debug(" P11SHUTTER IS OPEN")
            value = self.VALUES.OPEN
        else:
            self.log.debug(" P11SHUTTER IS CLOSED")
            value = self.VALUES.CLOSED

        # else:
        #     if time.time() - self.cmd_started > self.cmd_timeout:
        #        value = self.VALUES.UNKNOWN
        #     else:
        #        value = self.VALUES.MOVING

        self.update_value(value)

        self.log.debug("  update shutter state done")
        return value

    def simulated_update(self):
        if self.simulated_moving:
            value = self.VALUES.MOVING
        elif self.simulated_opened:
            value = self.VALUES.OPEN
        else:
            value = self.VALUES.CLOSED

        self.update_value(value)

        return value



from SpecClient_gevent.SpecMotor import SpecMotorA

from mxcubecore.BaseHardwareObjects import HardwareObject


class SpecMotor(HardwareObject, SpecMotorA):
    (NOTINITIALIZED, UNUSABLE, READY, MOVESTARTED, MOVING, ONLIMIT) = (0, 1, 2, 3, 4, 5)

    def __init__(self, name):
        super().__init__(name)
        SpecMotorA.__init__(self)

    def _init(self):
        SpecMotorA.connectToSpec(self, self.specname, self.specversion)

    def connect_notify(self, signal):
        if self.connection.isSpecConnected():
            if signal == "stateChanged":
                self.motorStateChanged(self.get_state())
            elif signal == "limitsChanged":
                self.motorLimitsChanged()
            elif signal == "valueChanged":
                self.motor_positions_changed(self.get_value())

    def motorStateChanged(self, state):
        self.set_is_ready(state > SpecMotor.UNUSABLE)

        self.emit("stateChanged", (state,))

    def motorIsMoving(self):
        return not self._ready_state_event.is_set()
        # return self.get_state() in (SpecMotor.MOVESTARTED, SpecMotor.MOVING)

    def motorLimitsChanged(self):
        self.emit("limitsChanged", (self.get_limits(),))

    def motorMoveDone(self, channelValue):
        SpecMotorA.motorMoveDone(self, channelValue)

        # print "motor state is ready ? %s (%s)" %
        # ((self.get_state()==SpecMotor.READY), self.get_state())
        if self.get_state() == SpecMotor.READY:
            self.emit("moveDone", (self.specversion, self.specname))

    def motor_positions_changed(self, absolutePosition):
        self.emit("valueChanged", (absolutePosition,))

    def syncQuestionAnswer(self, specSteps, controllerSteps):
        pass  # return '0' #NO ('1' means YES)

    def get_motor_mnemonic(self):
        return self.specName


class SpecVersionMotor(SpecMotor):
    def __init__(self, specversion, specname, username):
        super().__init__(specname)
        self.specversion = specversion
        self.specname = specname
        self.username = username
        SpecMotor.__init__(self, "internal" + username)
        self._init()



import logging

from mxcubecore import HardwareRepository as HWR
from mxcubecore.BaseHardwareObjects import HardwareObject


class ALBAEnergy(HardwareObject):
    def __init__(self, *args):
        super().__init__(*args)
        self.energy_position = None
        self.wavelength_position = None

    def init(self):
        self.wavelength_hwobj = self.get_object_by_role("wavelength")

        HWR.beamline.energy.connect("valueChanged", self.energy_position_changed)
        self.wavelength_hwobj.connect("valueChanged", self.wavelength_position_changed)

    def is_ready(self):
        return True

    def get_value(self):
        if self.energy_position is None:
            self.energy_position = HWR.beamline.energy.get_value()
        return self.energy_position

    def get_wavelength(self):
        if self.wavelength_position is None:
            self.wavelength_position = self.wavelength_hwobj.get_value()
        return self.wavelength_position

    def re_emit_values(self):
        HWR.beamline.energy.re_emit_values()

    def energy_position_changed(self, value):
        self.energy_position = value
        if None not in [self.energy_position, self.wavelength_position]:
            self.emit("energyChanged", self.energy_position, self.wavelength_position)

    def wavelength_position_changed(self, value):
        self.wavelength_position = value
        if None not in [self.energy_position, self.wavelength_position]:
            self.emit("energyChanged", self.energy_position, self.wavelength_position)

    def set_value(self, value):
        current_egy = self.get_value()

        logging.getLogger("HWR").debug(
            "moving energy to %s. now is %s" % (value, current_egy)
        )
        HWR.beamline.energy.set_value(value)

    def wait_move_energy_done(self):
        HWR.beamline.energy.wait_end_of_move()

    def set_wavelength(self, value):
        self.wavelength_hwobj.set_value(value)

    def get_limits(self):
        return HWR.beamline.energy.get_limits()

    def get_wavelength_limits(self):
        return self.wavelength_hwobj.get_limits()


def test_hwo(hwo):

    print("Wavelength is: ", hwo.get_wavelength())
    print("Energy limits are: ", hwo.get_limits())
    print("Wavelength limits are: ", hwo.get_wavelength_limits())


if __name__ == "__main__":
    test()



#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

""" Mockup shutter implementation"""

from enum import (
    Enum,
    unique,
)

from mxcubecore.BaseHardwareObjects import HardwareObjectState
from mxcubecore.HardwareObjects.abstract.AbstractShutter import AbstractShutter
from mxcubecore.HardwareObjects.mockup.ActuatorMockup import ActuatorMockup


@unique
class ShutterStates(Enum):
    """Shutter states definitions."""

    OPEN = HardwareObjectState.READY, 6
    CLOSED = HardwareObjectState.READY, 7
    MOVING = HardwareObjectState.BUSY, 8
    DISABLED = HardwareObjectState.WARNING, 9
    AUTOMATIC = HardwareObjectState.READY, 10


class ShutterMockup(AbstractShutter, ActuatorMockup):
    """
    ShutterMockup for simulating a simple open/close shutter.
    Fake some of the states of the shutter to correspong to values.
    """

    SPECIFIC_STATES = ShutterStates

    def init(self):
        """Initialisation"""
        super().init()
        self._initialise_values()
        self.update_value(self.VALUES.CLOSED)
        self.update_state(self.STATES.READY)

    def _initialise_values(self):
        """Add additional, known in advance states to VALUES"""
        values_dict = {item.name: item.value for item in self.VALUES}
        values_dict.update(
            {
                "MOVING": "In Motion",
                "UNUSABLE": "Temporarily not controlled",
            }
        )
        self.VALUES = Enum("ValueEnum", values_dict)




from mxcubecore.BaseHardwareObjects import HardwareObject


class RobodiffFShut(HardwareObject):
    def __init__(self, name):
        super().__init__(name)

    def init(self):
        self.robodiff = self.get_object_by_role("robot")
        self.connect(self.robodiff.controller.fshut, "state", self.value_changed)
        self.actuatorState = "unknown"

    def connect_notify(self, signal):
        if signal == "actuatorStateChanged":
            self.get_actuator_state(read=True)

    def value_changed(self, value):
        if value == "CLOSED":
            self.actuatorState = "out"
        elif value == "OPENED":
            self.actuatorState = "in"
        else:
            self.actuatorState = "unknown"
        self.emit("actuatorStateChanged", (self.actuatorState,))
        self.emit("actuatorStateChanged", (self.actuatorState,))

    def getWagoState(self, *args):
        return self.get_actuator_state(self, *args)

    def get_actuator_state(self, read=False):
        if read:
            self.value_changed(self.robodiff.controller.fshut.state())
        return self.actuatorState

    def actuatorIn(self, wait=True, timeout=None):
        self.robodiff.controller.fshut.open()
        self.get_actuator_state(read=True)

    def wagoIn(self):
        return self.actuatorIn()

    def actuatorOut(self, wait=True, timeout=None):
        self.robodiff.controller.fshut.close()
        self.get_actuator_state(read=True)

    def wagoOut(self):
        return self.actuatorOut()



import os
import sys

from mxcubecore.BaseHardwareObjects import HardwareObject


class ObjectsController(HardwareObject):
    def __init__(self, *args):
        HardwareObject.__init__(self, *args)

    def init(self, *args):
        sys.path.insert(0, self.get_property("source"))
        config = __import__("config", globals(), locals(), [])

        cfg_file = os.path.join(
            self.get_property("source"), self.get_property("config_file")
        )
        config.load(cfg_file)
        objects = config.get_context_objects("default")

        for obj_name, obj in objects.items():
            setattr(self, obj_name, obj)

    def centrebeam(self):
        self.robot.centrebeam()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
 
"""Transmission control for MANACA.
   Attenuator: ABS-300-12-C-DN-DN40 Flanges.
   Foils: #1, 8 um Al;
          #2, 10 um Al;
          #3, 20 um Al;
          #4, 80 um Al;
          #5, 160 um Al;
          #6, 320 um Al;
          #7, 800 um Al;
          #8, 1500 um Al;
          #9, 8 um Ti;
          #10, 10 um Cu
          #11, 5 um Au;
          #12, 25 um Zr.
    Here we use the Beer-Lambert law to calculate the transmission/attenuation: I = I0 * e^-(MU * d)
    were I is the transmitted (final) beam intensity, I0 is the incident (initial) beam intensity,
    MU is the linear attenuation coeficient (cm^-1) and d is the thickness of the foil (cm).
"""
import argparse
import itertools
import math
import time

from epics import PV


def read_input():
    # Read inputs from terminal
    ap = argparse.ArgumentParser()
    ap.add_argument("-e", "--energy", required=True, type=float, help="Energy of the x-ray beam [keV]")
    ap.add_argument("-t", "--transmission", required=True, type=float, help="Transmission to be set, [%]")
    args = vars(ap.parse_args())
    return args

def get_transmission(energy, transmission):
    transmission = transmission/100
    # Expressions (interpolation) for calculations of MU (linear attenuation coeficient [cm^-1]) for each material in function of the beam energy. 
    # The data used to obtain the expressions (curve fitting) are from  "https://physics.nist.gov/PhysRefData/FFast/html/form.html"
    # This attenuation coeficients can be verified and optimised after experimental validation at the beamline.
    MU_Al = (78657.01011 * math.exp(-energy/0.65969)) + (6406.36151 * math.exp(-energy/1.63268)) + (492.29999 * math.exp(-energy/4.42554)) + (3.2588)
    MU_Ti = (28062.17632 * math.exp(-energy/1.88062)) + (6354120 * math.exp(-energy/0.49973)) + (2257.91488 * math.exp(-energy/5.28296)) + (17.4342)
    MU_Cu = (1147850000 * math.exp(-energy/0.56933)) + (2582.7593 * math.exp(-energy/8.40671)) + (30628.08291 * math.exp(-energy/2.98937)) + (22.4187)
    MU_Au = (42783.04258 * math.exp(-energy/5.39244)) + (399.59714)
    MU_Zr = (11143.28458 * math.exp(-energy/5.87029)) + (102.27474)
    MU_EMPTY = 0.0
    # Foils in ABS-attenuator. The dictionary gives the foil position and material in the key (e.g. 'F1_Al', position 1 occuped by an aluminium foil),
    # and the thickness [um] in the value. The list gives the positions and materials, similar to the dictionary, and it is used in the combination. 
    foils_dict = {'F0_EMPTY':0, 'F1_Al':8, 'F2_Al':10, 'F3_Al':20, 'F4_Al':80, 'F5_Al':160, 'F6_Al':320, 'F7_Al':800,
                  'F8_Al':1500, 'F9_Ti':8, 'F10_Cu':10, 'F11_Au':5, 'F12_Zr':25}

    attenuator_position = ['F0_EMPTY', 'F1_Al', 'F2_Al', 'F3_Al', 'F4_Al', 'F5_Al', 'F6_Al', 'F7_Al', 'F8_Al',
                           'F9_Ti', 'F10_Cu', 'F11_Au', 'F12_Zr']
    # conditions to use only the continous part of the absorption spectra and not the edge region. It can be optimised after experimental validation.
    if energy < 6.5:
        attenuator_position = attenuator_position[0:9]
    elif 6.5 <= energy < 9.5 :
        attenuator_position = attenuator_position[0:10]
    elif 9.5 <= energy < 15.0 :
        attenuator_position = attenuator_position[0:11]
    elif 15.0 <= energy < 18.5 :
        attenuator_position = attenuator_position[0:12]
    elif energy >= 18.5:
        attenuator_position = attenuator_position[0:13]
    
    foils_comb = []
    atten_coef_sum = []
    
    # calculate the all possible unique combinations
    for i in range(1, len(attenuator_position)+1):
        product = itertools.combinations(attenuator_position, i)
        for item in product:
            foils_comb.append(list(item))
    # calculate the product of MU (cm^-1) * thickness (cm) for each material (MU * d)
    # and sum the products to obtain the total (final) attenuation for each possible combination.
    # and put the results (total attenuation) in a list. 
    comb_sum = []
    for item in foils_comb:
        comb_tmp = []
        for sub_item in item:
            if '_Al' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_Al
                comb_tmp.append(atten_mu_x)
            elif '_Ti' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_Ti
                comb_tmp.append(atten_mu_x)
            elif '_Cu' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_Cu
                comb_tmp.append(atten_mu_x)
            elif '_Au' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_Au
                comb_tmp.append(atten_mu_x)
            elif '_Zr' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_Zr
                comb_tmp.append(atten_mu_x)
            elif '_EMPTY' in sub_item:
                atten_mu_x = float(foils_dict[sub_item] * 1.0E-4) * MU_EMPTY
                comb_tmp.append(atten_mu_x)

        comb_sum.append(sum(comb_tmp))

    # create a dictionary to correlate the combinations of foils (value) with the total attenuation (key)
    FOILS = dict(zip(comb_sum, foils_comb))
    # calculate the transmission for each combination using the product calculated before (the keys of FOILS)
    FOILS_T = {1.0 * (math.exp(-k)) : v for k, v in FOILS.items()}
    # find the closest transmission that is possible to obtain with the foils on ABS-300.
    closest_transmission = min(FOILS_T.keys(), key=lambda k:abs(float(k)-transmission))
    # get the foil combination for the closest transmission found
    filter_combination = FOILS_T[closest_transmission]

    return transmission, closest_transmission, filter_combination


def set_foils(filter_combination):
    
    # declare a list with available foils and a dictionary with epics PV class (FOIL ACT, FOIL IN, FOIL OUT) for each foil
    attenuator_position = ['F1_Al', 'F2_Al', 'F3_Al', 'F4_Al', 'F5_Al', 'F6_Al', 'F7_Al', 'F8_Al',
                           'F9_Ti', 'F10_Cu', 'F11_Au', 'F12_Zr']
    
    foils_pv = {'F1_Al':(PV('MNC:B:RIO01:9474C:bo0'), PV('MNC:B:RIO01:9425A:bi11'), PV('MNC:B:RIO01:9425A:bi12')),
                 'F2_Al':(PV('MNC:B:RIO01:9474C:bo1'), PV('MNC:B:RIO01:9425A:bi10'), PV('MNC:B:RIO01:9425A:bi13')),
                 'F3_Al':(PV('MNC:B:RIO01:9474C:bo2'), PV('MNC:B:RIO01:9425A:bi9'), PV('MNC:B:RIO01:9425A:bi14')),
                 'F4_Al':(PV('MNC:B:RIO01:9474C:bo3'), PV('MNC:B:RIO01:9425A:bi8'), PV('MNC:B:RIO01:9425A:bi15')),
                 'F5_Al':(PV('MNC:B:RIO01:9474C:bo4'), PV('MNC:B:RIO01:9425A:bi7'), PV('MNC:B:RIO01:9425A:bi16')),
                 'F6_Al':(PV('MNC:B:RIO01:9474C:bo5'), PV('MNC:B:RIO01:9425A:bi6'), PV('MNC:B:RIO01:9425A:bi17')),
                 'F7_Al':(PV('MNC:B:RIO01:9474C:bo6'), PV('MNC:B:RIO01:9425A:bi5'), PV('MNC:B:RIO01:9425A:bi18')),
                 'F8_Al':(PV('MNC:B:RIO01:9474C:bo7'), PV('MNC:B:RIO01:9425A:bi4'), PV('MNC:B:RIO01:9425A:bi19')),
                 'F9_Ti':(PV('MNC:B:RIO01:9474D:bo0'), PV('MNC:B:RIO01:9425A:bi3'), PV('MNC:B:RIO01:9425A:bi20')),
                 'F10_Cu':(PV('MNC:B:RIO01:9474D:bo1'), PV('MNC:B:RIO01:9425A:bi2'), PV('MNC:B:RIO01:9425A:bi21')),
                 'F11_Au':(PV('MNC:B:RIO01:9474D:bo2'), PV('MNC:B:RIO01:9425A:bi1'), PV('MNC:B:RIO01:9425A:bi22')),
                 'F12_Zr':(PV('MNC:B:RIO01:9474D:bo3'), PV('MNC:B:RIO01:9425A:bi0'), PV('MNC:B:RIO01:9425A:bi23'))}

    # setup the foils: check foil status, put the foils in the 'filter_combination' and remove the others
    
    wt = 0.1
    
    # the commented lines bellow works for the attenuator expected logic (0 for foil out and 1 for foil in).
    # as the current logic is inverted (1 is foil out and 0 is foil in) use the uncommented lines.

    for foil in attenuator_position:
        if foil in filter_combination:
            # if foils_pv[foil][1].get() == 0 and foils_pv[foil][2].get() == 1:
            if foils_pv[foil][1].get() == 1 and foils_pv[foil][2].get() == 0:
                # foils_pv[foil][0].put(1)
                foils_pv[foil][0].put(0)
                status = 0
                time.sleep(wt)
            # elif foils_pv[foil][1].get() == 1 and foils_pv[foil][2].get() == 0:
            elif foils_pv[foil][1].get() == 0 and foils_pv[foil][2].get() == 1:
                status = 0
            elif foils_pv[foil][1].get() == foils_pv[foil][2].get():
                status = 1
                break
        else:
            # if foils_pv[foil][1].get() == 0 and foils_pv[foil][2].get() == 1:
            if foils_pv[foil][1].get() == 1 and foils_pv[foil][2].get() == 0:
                status = 0
            # elif foils_pv[foil][1].get() == 1 and foils_pv[foil][2].get() == 0:
            elif foils_pv[foil][1].get() == 0 and foils_pv[foil][2].get() == 1:
                # foils_pv[foil][0].put(0)
                foils_pv[foil][0].put(1)
                status = 0
                time.sleep(wt)
            elif foils_pv[foil][1].get() == foils_pv[foil][2].get():
                status = 1
                break
    
    return status


def main():
    energy = read_input()['energy']
    transmission = read_input()['transmission']
    transmission_setup = get_transmission(energy, transmission)
    # transmission required by the user
    user_transmission = transmission_setup[0] * 100
    # real transmission got with calculated foil combination 
    actual_transmission = round(transmission_setup[1] * 100, 2)
    # calculated foil combination to get the required transmission
    filter_combination = transmission_setup[2]
    # put the foils and check if the they were correctly positioned (status:0 = ok; 1 = fail)
    foil_status = set_foils(filter_combination)

if __name__ == "__main__":
    main()



class Component(object):
    """
    Entity class representing any a sample or sample container
    """

    def __init__(self, container, address, scannable):
        self.container = container
        self.address = address
        self.scannable = scannable
        self.id = None
        self.present = False
        self.selected = False
        self.scanned = False
        self.dirty = False
        self._leaf = False
        self._name = ""

    #########################           PUBLIC           #########################

    def get_name(self):
        return self._name

    def get_id(self):
        """
        Returns a unique ID of an element - typically scanned from the real object
        Can be None if sample is unknown or not present
        :rtype: str
        """
        return self.id

    def get_address(self):
        """
        Returns a unique identifier of the slot of the element ()
        Can never be None - even if the component is not present
        :rtype: str
        """
        return self.address

    def get_coords(self):
        coords_list = [self.get_index() + 1]
        x = self.get_container()
        while x:
            idx = x.get_index()
            if idx is not None:
                coords_list.append(idx + 1)
            x = x.get_container()
        coords_list.reverse()
        return tuple(coords_list)

    def get_index(self):
        """
        Returns the index of the object within the parent's component list,
        :rtype: int
        """
        try:
            container = self.get_container()
            if container is not None:
                components = container.get_components()
                for i in range(len(components)):
                    if components[i] is self:
                        return i
        except Exception:
            return -1

    def is_leaf(self):
        return self._leaf

    def is_present(self):
        """
        Returns true if the element is known to be currently present
        :rtype: bool
        """
        return self.present

    def is_selected(self):
        """
        Returns if the element is currently selected
        :rtype: bool
        """
        return self.selected

    def is_scanned(self):
        """
        Returns if the element has been scanned for ID (for scannable components)
        :rtype: bool
        """
        if self.is_scannable() == False:
            return False
        return self.scanned

    def is_scannable(self):
        """
        Returns if the element can be scanned for ID
        :rtype: bool
        """
        return self.scannable

    def assert_is_scannable(self):
        if not self.is_scannable():
            raise "Element is not scannable"

    def get_container(self):
        """
        Returns the parent of this element
        :rtype: Container
        """
        return self.container

    def get_siblings(self):
        """
        Returns the parent of this element
        :rtype: Container
        """
        ret = []
        if self.get_container() is not None:
            for c in self.get_container().get_components():
                if c != self:
                    ret.append(c)
        return ret

    def clear_info(self):
        """
        Clears all sample info (also in components if object is a container)
        """
        changed = False
        if self.id is not None:
            self.id = None
            changed = True
        if self.present:
            self.present = False
            changed = True
        if self.scanned:
            self.scanned = False
            changed = True
        if changed:
            self._set_dirty()

    #########################           PROTECTED           #########################
    def _set_info(self, present=False, id=None, scanned=False):
        changed = False
        if self.id != id:
            self.id = id
            changed = True
        if self.id:
            present = True
        if self.present != present:
            self.present = present
            changed = True
        if self.is_scannable() == False:
            scanned = False
        if self.scanned != scanned:
            self.scanned = scanned
            changed = True
        if changed:
            self._set_dirty()

    def _set_selected(self, selected):
        if selected:
            for c in self.get_siblings():
                c._set_selected(False)
            if self.get_container() is not None:
                self.get_container()._set_selected(True)
        self.selected = selected

    def _is_dirty(self):
        return self.dirty

    def _set_dirty(self):
        self.dirty = True
        container = self.get_container()
        if container is not None:
            container._set_dirty()

    def _reset_dirty(self):
        self.dirty = False



#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

"""
[Name]
ALBAMachineInfo

[Description]
Hardware Object is used to get relevant machine information
(machine current, time to next injection, status, etc)
Based on EMBL HwObj

[Channels]
- MachineCurrent
- TopUpRemaining
- State

[Commands]

[Emited signals]
- valuesChanged

[Functions]
- None

[Included Hardware Objects]
- None


Example Hardware Object XML file :
==================================
<object class="ALBAMachineInfo">
    <username>Mach</username>
    <taurusname>mach/ct/gateway</taurusname>
    <channel type="sardana" name="MachStatus" polling="1000">State</channel>
    <channel type="sardana" name="MachCurrent" polling="1000">Current</channel>
    <channel type="sardana" name="TopUpRemaining" polling="1000">TopUpRemaining</channel>
</object>
"""

import logging
import time
from datetime import (
    datetime,
    timedelta,
)

from gevent import spawn
from urllib2 import urlopen

from mxcubecore import HardwareRepository as HWR
from mxcubecore.BaseHardwareObjects import HardwareObject

__author__ = "Jordi Andreu"
__credits__ = ["MXCuBE collaboration"]

__version__ = "2.2."
__maintainer__ = "Jordi Andreu"
__email__ = "jandreu[at]cells.es"
__status__ = "Draft"


class ALBAMachineInfo(HardwareObject):
    """
    Descript. : Displays actual information about the machine status.
    """

    def __init__(self, name):
        super().__init__(name)
        self.logger = logging.getLogger("HWR MachineInfo")
        self.logger.info("__init__()")

        """
        Descript. :
        """
        # Parameters values
        self.values_dict = {}
        self.values_dict["mach_current"] = None
        self.values_dict["mach_status"] = ""
        self.values_dict["topup_remaining"] = ""
        # Dictionary for booleans indicating if values are in range
        #        self.values_in_range_dict = {}
        self.chan_mach_current = None
        self.chan_mach_status = None
        self.chan_topup_remaining = None

    def init(self):
        """
        Descript. : Inits channels from xml configuration.
        """
        try:
            self.chan_mach_current = self.get_channel_object("MachCurrent")
            if self.chan_mach_current is not None:
                self.chan_mach_current.connect_signal(
                    "update", self.mach_current_changed
                )

            self.chan_mach_status = self.get_channel_object("MachStatus")
            if self.chan_mach_status is not None:
                self.chan_mach_status.connect_signal("update", self.mach_status_changed)

            self.chan_topup_remaining = self.get_channel_object("TopUpRemaining")
            if self.chan_topup_remaining is not None:
                self.chan_topup_remaining.connect_signal(
                    "update", self.topup_remaining_changed
                )
        except KeyError:
            self.logger.warning("%s: cannot read machine info", self.name())

    def mach_current_changed(self, value):
        """
        Descript. : Function called if the machine current is changed
        Arguments : new machine current (float)
        Return    : -
        """
        if (
            self.values_dict["mach_current"] is None
            or abs(self.values_dict["mach_current"] - value) > 0.10
        ):
            self.values_dict["mach_current"] = value
            self.re_emit_values()
            self.logger.debug("New machine current value=%smA" % value)

    def mach_status_changed(self, status):
        """
        Descript. : Function called if machine status is changed
        Arguments : new machine status (string)
        Return    : -
        """
        self.values_dict["mach_status"] = str(status)
        self.re_emit_values()
        self.logger.debug("New machine status=%s" % status)

    def topup_remaining_changed(self, value):
        """
        Descript. : Function called if topup ramaining is changed
        Arguments : new topup remainin (float)
        Return    : -
        """
        self.values_dict["topup_remaining"] = value
        self.re_emit_values()
        self.logger.debug("New top-up remaining time=%ss" % value)

    def re_emit_values(self):
        """
        Descript. : Updates storage disc information, detects if intensity
                    and storage space is in limits, forms a value list
                    and value in range list, both emited by qt as lists
        Arguments : -
        Return    : -
        """

        values_to_send = []
        values_to_send.append(self.values_dict["mach_current"])
        values_to_send.append(self.values_dict["mach_status"])
        values_to_send.append(self.values_dict["topup_remaining"])

        self.emit("valuesChanged", values_to_send)
        self.logger.debug("SIGNAL valuesChanged emitted")

    def get_mach_current(self):
        try:
            value = self.chan_mach_current.get_value()
        except Exception as e:
            self.logger.error("Cannot read machine current value, returning 0")
            value = 0
        finally:
            return value
            # return self.values_dict['mach_current']

    def get_current(self):
        return self.get_mach_current()

    def get_message(self):
        return "Machinfo status: %s" % self.get_mach_status()

    #    def get_current_value(self):
    #        """
    #        Descript. :
    #        """
    #        return self.values_dict['current']

    def get_mach_status(self):
        return self.chan_mach_status.get_value()

    #        return self.values_dict['mach_status']

    def get_topup_remaining(self):
        return self.chan_topup_remaining.get_value()


#        return self.values_dict['remaining']


def test_hwo(hwo):
    print("Current is", hwo.get_current())



import abc
import logging
import os
import time

import gevent

from mxcubecore.BaseHardwareObjects import HardwareObject
from mxcubecore.TaskUtils import error_cleanup


class AbstractEnergyScan(HardwareObject):
    __metaclass__ = abc.ABCMeta

    def __init__(self, name):
        super().__init__(name)
        self.data_collect_task = None
        self._egyscan_task = None
        self.scanning = False
        self.cpos = None

    def open_safety_shutter(self, timeout):
        """
        Open the safety shutter. Give a timeout [s] if needed.
        """
        pass

    def close_safety_shutter(self, timeout):
        """
        Close the safety shutter. Give a timeout [s] if needed.
        """
        pass

    def open_fast_shutter(self):
        """
        Open the fast shutter.
        """
        pass

    def close_fast_shutter(self):
        """
        Close the fast shutter.
        """
        pass

    def energy_scan_hook(self, energy_scan_parameters):
        """
        Execute actions, required before running the raw scan(like changing
        undulator gaps, move to a given energy... These are in general
        beamline specific actions.
        """
        pass

    def execute_energy_scan(self, energy_scan_parameters):
        """
        Execute the raw scan sequence. Here is where you pass whatever
        parameters you need to run the raw scan (e.g start/end energy,
        counting time, energy step...).
        """
        pass

    def get_static_parameters(self, config_file, element, edge):
        """
        Get any parameters, which are known before hand. Some of them are
        known from the theory, like the peak energy, others are equipment
        specific like the lower/upper ROI limits of the fluorescence detector.
        Usually these parameters are pre-defined in a file, but can also be
        calculated.
        The function should return a distionary with at least defined
        {'edgeEnergy': peak_energy} member, where 'edgeEnergy' is a
        compulsory key.
        It is convinient to put in the same dictionary the remote energy,
        the ROI min/max values.
        There are few more reserved key names:
        'eroi_min', 'eroi_max' - min and max ROI limits if you want ot set one.
        'findattEnergy' - energy to move to if you want to choose the attenuation
        for the scan.
        """
        pass

    def set_mca_roi(self, eroi_min, eroi_max):
        """
        Configure the fluorescent detector ROI. The input is min/max energy.
        """
        pass

    def calculate_und_gaps(self, energy):
        """
        Calculate the undulator(s) gap(s), If specified, undulator is the
        name of the undulator to chose if several possibilities. Return
        a dictionary undulator:gap in the order undulator(s)should move.
        """
        pass

    def move_undulators(self, undulators):
        """
        Move the undulator(s) to gap(s), where undulators is a dictionary
        undulator:gap and should be in the order of which motor to move first.
        """
        pass

    def escan_prepare(self):
        """
        Set the nesessary equipment in position for the scan. No need to know the c=scan paramets.
        """
        pass

    def choose_attenuation(self):
        """
        Procedure to set the minimal attenuation in order no preserve
        the sample. Should be done at the energy after the edge.
        """
        pass

    def move_energy(self, energy):
        """
        Move the monochromator to energy - used before and after the scan.
        """
        pass

    def escan_cleanup(self):
        pass

    def escan_postscan(self):
        """
        set the nesessary equipment in position after the scan
        """
        pass

    # def do_energy_scan(self):
    def do_energy_scan(self):
        with error_cleanup(self.escan_cleanup):
            self.escan_prepare()
            self.energy_scan_hook(self.energy_scan_parameters)
            self.open_safety_shutter(timeout=10)
            self.choose_attenuation()
            self.close_fast_shutter()
            logging.getLogger("HWR").debug("Doing the scan, please wait...")
            self.execute_energy_scan(self.energy_scan_parameters)
            self.escan_postscan()
            self.close_fast_shutter()
            self.close_safety_shutter(timeout=10)
            # send finish sucessfully signal to the brick
            self.emit("energyScanFinished", (self.energy_scan_parameters,))
            self.ready_event.set()

    # def start_energy_scan(
    def start_energy_scan(
        self,
        element,
        edge,
        directory,
        prefix,
        session_id=None,
        blsample_id=None,
        cpos=None,
    ):
        if self._egyscan_task and not self._egyscan_task.ready():
            raise RuntimeError("Scan already started.")

        self.emit("energyScanStarted", ())
        # Set the energy from the element and edge parameters
        STATICPARS_DICT = self.get_static_parameters(
            self.get_property("config_file"), element, edge
        )
        self.cpos = cpos
        if STATICPARS_DICT is not None:
            self.energy_scan_parameters = STATICPARS_DICT
        else:
            self.energy_scan_parameters = {}
        self.energy_scan_parameters["element"] = element
        self.energy_scan_parameters["edge"] = edge
        self.energy_scan_parameters["directory"] = directory

        # Calculate the MCA ROI (if needed)
        try:
            self.set_mca_roi(STATICPARS_DICT["eroi_min"], STATICPARS_DICT["eroi_max"])
        except Exception:
            pass

        # Calculate undulator gaps (if any)
        GAPS = {}
        try:
            GAPS = self.calculate_und_gaps(STATICPARS_DICT["edgeEnergy"])
        except Exception:
            pass

        # create the directory if needed
        if not os.path.exists(directory):
            os.makedirs(directory)
        self.energy_scan_parameters["prefix"] = prefix
        if session_id is not None:
            self.energy_scan_parameters["sessionId"] = session_id
            self.energy_scan_parameters["blSampleId"] = blsample_id
            self.energy_scan_parameters["startTime"] = time.strftime(
                "%Y-%m-%d %H:%M:%S"
            )

        self._egyscan_task = gevent.spawn(self.do_energy_scan)

    def do_chooch(self, elememt, edge, scanArchiveFilePrefix, scanFilePrefix):
        """
        Use chooch to calculate edge and inflection point
        The brick expects the folowing parameters to be returned:
        pk, fppPeak, fpPeak, ip, fppInfl, fpInfl, rm,
        chooch_graph_x, chooch_graph_y1, chooch_graph_y2, title)
        """
        pass

    def enable_max_transmission(self, state):
        """
        Enables/disables usage of maximal transmission set
        during the energy scan
        """
        pass

    def set_max_transmission(self, value):
        """
        Sets maximal transmission used during the energy scan
        """
        pass

    def adjust_transmission(self, value):
        pass

    def get_adjust_transmission_state(self):
        return

    def get_max_transmission_value(self):
        return



"""
EPICS implementation of AbstractMotor.

Example of xml file:

<object class="LNLS.EPICSMotor">
    <channel type="epics" name="epicsActuator_val">SOL:S:m1.VAL</channel>
    <channel type="epics" name="epicsActuator_rbv" polling="500">SOL:S:m1.RBV</channel>
    <channel type="epics" name="epicsMotor_rlv">SOL:S:m1.RLV</channel>
    <channel type="epics" name="epicsMotor_dmov" polling="500">SOL:S:m1.DMOV</channel>
    <channel type="epics" name="epicsMotor_stop">SOL:S:m1.STOP</channel>
    <channel type="epics" name="epicsMotor_velo">SOL:S:m1.VELO</channel>
    <channel type="epics" name="epicsMotor_llm">SOL:S:m1.DLLM</channel>
    <channel type="epics" name="epicsMotor_hlm">SOL:S:m1.DHLM</channel>
    <channel type="epics" name="epicsMotor_egu">SOL:S:m1.EGU</channel>
    <username>Omega</username>
    <motor_name>Omega</motor_name>
    <unit>1e-3</unit>
    <GUIstep>90</GUIstep>
</object>
"""
import logging
import time

import gevent

from mxcubecore.HardwareObjects.abstract.AbstractMotor import AbstractMotor
from mxcubecore.HardwareObjects.LNLS.EPICSActuator import EPICSActuator


class EPICSMotor(EPICSActuator, AbstractMotor):
    """EPICS Motor class"""

    MOTOR_DMOV = 'epicsMotor_dmov'
    MOTOR_STOP = 'epicsMotor_stop'
    MOTOR_RLV = 'epicsMotor_rlv'
    MOTOR_VELO = 'epicsMotor_velo'
    MOTOR_HLM = 'epicsMotor_hlm'
    MOTOR_LLM = 'epicsMotor_llm'
    MOTOR_EGU = 'epicsMotor_egu'

    def __init__(self, name):
        super().__init__(name)
        self._wrap_range = None

    def init(self):
        """ Initialization method """
        super().init()
        self.get_limits()
        self.get_velocity()
        self.__watch_task = gevent.spawn(self._watch)
        self.update_state(self.STATES.READY)
    
    def _watch(self):
        """ Watch motor current value and update it on the UI."""
        while True:
            time.sleep(0.25)
            self.update_value()

    def _wait_actuator(self):
        """Override EPICSActuator method."""
        self.update_specific_state(self.SPECIFIC_STATES.MOVING)
        current_value = self.get_value()

        while (not self.done_movement()):
            time.sleep(0.25)

        self.update_specific_state(None)
        self.update_state(self.STATES.READY)

    def abort(self):
        """Override EPICSActuator method."""
        self.set_channel_value(self.MOTOR_STOP, 1)
        super().abort()

    def get_limits(self):
        """Override AbstractActuator method."""
        try:
            low_limit = float(self.get_channel_value(self.MOTOR_LLM))
            high_limit = float(self.get_channel_value(self.MOTOR_HLM))
            self._nominal_limits = (low_limit, high_limit)
        except BaseException:
            self._nominal_limits = (None, None)

        if self._nominal_limits in [(0, 0), (float('-inf'), float('inf'))]:
            # Treat infinite limits
            self._nominal_limits = (None, None)
        
        logging.getLogger("HWR").info('Motor %s limits: %s' % (self.motor_name, self._nominal_limits))
        return self._nominal_limits

    def get_velocity(self):
        """Override AbstractMotor method."""
        self._velocity = self.get_channel_value(self.MOTOR_VELO)
        return self._velocity

    def set_velocity(self, value):
        """Override AbstractMotor method."""
        self.__velocity = self.set_channel_value(self.MOTOR_VELO, value)
    
    def done_movement(self):
        """ Return whether motor finished movement or not."""
        dmov = self.get_channel_value(self.MOTOR_DMOV)
        return bool(dmov)




# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU General Lesser Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.
"""Transmission with bliss """

from mxcubecore.HardwareObjects.abstract.AbstractTransmission import (
    AbstractTransmission,
)

__copyright__ = """ Copyright © 2020 by the MXCuBE collaboration """
__license__ = "LGPLv3+"


class Transmission(AbstractTransmission):
    """Transmission class"""

    unit = "%"

    def __init__(self, name):
        super(Transmission, self).__init__(name)
        self._transmission = None

    def init(self):
        """Initialise from the config"""
        super(Transmission, self).init()
        module_name = self.get_property("module_name")
        ctrl = self.get_object_by_role("controller")
        self._transmission = getattr(ctrl, module_name)

    def _set_value(self, value):
        """Set the transmission.
        Args:
            value(float): Transmission [%]
        """
        self.update_state(self.STATES.BUSY)
        self._transmission.set(value)
        self.update_state(self.STATES.READY)

    def get_value(self):
        """Get the real transmission value
        Returns:
            (float): Transmission [%]
        """
        return self._transmission.get()



# encoding: utf-8
#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU General Lesser Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

"""
bliss implementation of AbstartNState
Example xml file:
<object class="BlissNState">
  <username>Detector Cover</username>
  <actuator_name>detcover</>
  <object href="/bliss" role="controller"/>
  <values>{"IN": "IN", "OUT": "OUT"}</values>
</object>
"""
from enum import Enum

from mxcubecore.HardwareObjects.abstract.AbstractNState import (
    AbstractNState,
    BaseValueEnum,
)
from mxcubecore.HardwareObjects.BlissMotor import BlissMotor

__copyright__ = """ Copyright © 2020 by the MXCuBE collaboration """
__license__ = "LGPLv3+"


class BlissNState(AbstractNState):
    """bliss implementation of AbstartNState"""

    def __init__(self, name):
        super().__init__(name)
        self._bliss_obj = None
        self.device_type = None
        self.__saved_state = None
        self._prefix = None

    def init(self):
        """Initialise the device"""

        super().init()
        self._prefix = self.get_property("prefix")
        self._bliss_obj = getattr(
            self.get_object_by_role("controller"), self.actuator_name
        )

        self.device_type = self.get_property("type", "actuator")
        if "MultiplePositions" in self._bliss_obj.__class__.__name__:
            self.device_type = "motor"

        self.initialise_values()
        self.__saved_state = self.get_value().value
        if self.device_type == "actuator":
            self.connect(self._bliss_obj, "state", self._update_value)
            self.connect(self._bliss_obj, "state", self._update_state)
        elif self.device_type == "motor":
            self.connect(self._bliss_obj, "position", self._update_value)
            self.connect(self._bliss_obj, "state", self._update_state_motor)

        self.update_value()
        self.update_state()

    # NB: Bliss calls the update handler with the state so its needed in the
    # method definition
    def _update_state(self, state=None):
        self.update_state(self.STATES.READY)

    def _update_value(self, value=None):
        if value:
            self.update_value(self.value_to_enum(value))
        else:
            self.update_value()

    def get_value(self):
        """Get the device value
        Returns:
            (Enum): Enum member, corresponding to the value or UNKNOWN.
        """
        if self.device_type == "motor":
            return self.value_to_enum(self._bliss_obj.position)

        if self.device_type == "actuator":
            if self._prefix:
                _attr = self._prefix + "_is_in"
                _cmd = getattr(self._bliss_obj, _attr)
                if isinstance(_cmd, bool):
                    _val = _cmd
                else:
                    _val = _cmd()
            else:
                _val = self._bliss_obj.state

        return self.value_to_enum(_val)

    def _set_value(self, value):
        """Set device to value.
        Args:
            value (str or enum): target value
        """
        self.update_state(self.STATES.BUSY)
        if isinstance(value, Enum):
            self.__saved_state = value.name
            if isinstance(value.value, (tuple, list)):
                svalue = value.value[0]
            else:
                svalue = value.value
        else:
            self.__saved_state = value.upper()
        # are we sure we never get to need svalue below without setting it first?
        if self.device_type == "motor":
            self._bliss_obj.move(svalue, wait=False)
        elif self.device_type == "actuator":
            if self._prefix:
                _attr = self._prefix + "_" + value.name.lower()
            else:
                _attr = "set_" + svalue.lower()
            _cmd = getattr(self._bliss_obj, _attr)
            _cmd()

    def get_state(self):
        """Get the device state.
        Returns:
            (enum 'HardwareObjectState'): Device state.
        """
        try:
            _state = self._bliss_obj.state.upper()
        except (AttributeError, KeyError):
            return self.STATES.UNKNOWN

        if self.device_type == "motor":
            try:
                return BlissMotor.SPECIFIC_TO_HWR_STATE[_state]
            except KeyError:
                return self.STATES.UNKNOWN

        if _state in ("IN", "OUT"):
            if self.__saved_state == _state:
                return self.STATES.READY
            return self.STATES.BUSY
        return self.STATES.UNKNOWN

    def _update_state_motor(self, state):
        """Update the state for the motor type."""
        try:
            state = BlissMotor.SPECIFIC_TO_HWR_STATE[state.upper()]
        except KeyError:
            state = self.STATES.UNKNOWN
        return self.update_state(state)

    def initialise_values(self):
        """Get the predefined valies. Create the VALUES Enum
        Returns:
            (Enum): "ValueEnum" with predefined values.
        """
        if self.device_type == "actuator":
            super().initialise_values()
        if self.device_type == "motor":
            try:
                values = {
                    val["label"].upper(): val["label"]
                    for val in self._bliss_obj.positions_list
                }
                self.VALUES = Enum(
                    "ValueEnum",
                    dict(values, **{item.name: item.value for item in BaseValueEnum}),
                )
            except AttributeError:
                super().initialise_values()



#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

from mxcubecore.HardwareObjects.EMBL.TINEMotor import TINEMotor

__credits__ = ["EMBL Hamburg"]
__license__ = "LGPLv3+"
__category__ = "Motor"


class EMBLResolution(TINEMotor):
    """
    Based on the TineMotor. After the move command executes additional
    commands.
    """

    def __init__(self, name):
        TINEMotor.__init__(self, name)

    def get_limits(self):
        return self._nominal_limits

    def get_value(self):
        return self._nominal_value



#! /usr/bin/env python
# encoding: utf-8
#
# This file is part of MXCuBE.
#
# MXCuBE is free software: you can redistribute it and/or modify
# it under the terms of the GNU Lesser General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# MXCuBE is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public License
# along with MXCuBE.  If not, see <https://www.gnu.org/licenses/>.
"""
"""

from __future__ import (
    absolute_import,
    division,
    print_function,
    unicode_literals,
)

import json
import logging
import os
import time

import requests

from mxcubecore import HardwareRepository as HWR
from mxcubecore.HardwareObjects.abstract.AbstractXrayCentring import (
    AbstractXrayCentring,
)

__copyright__ = """ Copyright © 2016 - 2022 by MXCuBE Collaboration """
__license__ = "LGPLv3+"
__author__ = "rhfogh"
__date__ = "25/03/2022"
__category__ = "General"


class ESRFSmallXrayCentring(AbstractXrayCentring):
    def execute(self):
        """Executes the BES workflow SmallXrayCentring"""

        logging.getLogger("HWR").debug("Executes SmallXrayCentring workflow")
        workflow_name = "SmallXrayCentring"
        bes_host = "mxbes3-2204"
        bes_port = 38180
        task_group_node_id = self._data_collection_group._node_id
        dict_parameters = json.loads(json.dumps(HWR.beamline.workflow.dict_parameters))
        dict_parameters["sample_node_id"] = task_group_node_id
        dict_parameters["end_workflow_in_mxcube"] = False
        dict_parameters["workflow_id"] = HWR.beamline.xml_rpc_server.workflow_id
        dict_parameters["workflow_parameters"] = (
            self._queue_entry.get_data_model().get_workflow_parameters()
        )
        logging.getLogger("HWR").info("Starting workflow {0}".format(workflow_name))
        logging.getLogger("HWR").info(
            "Starting a workflow on http://%s:%d/BES" % (bes_host, bes_port)
        )
        bes_url = "http://{0}:{1}".format(bes_host, bes_port)
        start_URL = os.path.join(bes_url, "RUN", workflow_name)
        logging.getLogger("HWR").info("BES start URL: %r" % start_URL)
        response = requests.post(start_URL, json=dict_parameters)
        if response.status_code == 200:
            request_id = response.text
            logging.getLogger("HWR").info(
                "Workflow started, request id: %r" % request_id
            )
        else:
            logging.getLogger("HWR").error("Workflow didn't start!")
            logging.getLogger("HWR").error(response.text)
        status_url = os.path.join(bes_url, "STATUS", str(request_id))
        logging.getLogger("HWR").info("STATUS URL: %r" % status_url)
        start_time = time.time()
        max_time = 600  # Max five minutes
        finished = False
        timed_out = False
        while not timed_out and not finished:
            response = requests.get(status_url)
            if response.text == "STARTED":
                logging.getLogger("HWR").info("Workflow still running...")
                time.sleep(1)
            elif response.text == "FINISHED":
                logging.getLogger("HWR").info("Workflow finished!")
                finished = True
            if time.time() > start_time + max_time:
                logging.getLogger("HWR").info("Workflow timed out!")
                timed_out = True



import math

import gevent

from mxcubecore.BaseHardwareObjects import HardwareObject
from mxcubecore.HardwareObjects.abstract.AbstractMotor import AbstractMotor


class GrobMotor(Device, AbstractMotor):
    (NOTINITIALIZED, UNUSABLE, READY, MOVESTARTED, MOVING, ONLIMIT) = (0, 1, 2, 3, 4, 5)

    def __init__(self, name):
        super().__init__(name)

    def init(self):
        self.motorState = GrobMotor.NOTINITIALIZED
        self.username = self.motor_name
        self.grob = self.get_object_by_role("grob")

        # this is ugly : I added it to make the centring procedure happy
        self.specName = self.motor_name

        self.motor = getattr(self.grob.controller, self.motor_name)
        self.connect(self.motor, "position", self.positionChanged)
        self.connect(self.motor, "state", self.updateState)

    def connect_notify(self, signal):
        if signal == "valueChanged":
            self.emit("valueChanged", (self.get_value(),))
        elif signal == "stateChanged":
            self.updateState()
        elif signal == "limitsChanged":
            self.motorLimitsChanged()

    def updateState(self, state=None):
        if state is None:
            state = self.motor.state()
            if isinstance(self.motor, self.grob.SampleTableMotor):
                if self.motor.is_unusable():
                    state = "UNUSABLE"
                elif self.motor.is_moving(state):
                    state = "MOVING"
                elif self.motor.is_on_limit(state):
                    state = "ONLIMIT"
                else:
                    state = "READY"
        # convert from grob state to Hardware Object motor state
        if state == "MOVING":
            state = GrobMotor.MOVING
        elif state == "READY" or state.startswith("WAIT_GET"):
            state = GrobMotor.READY
        elif state == "ONLIMIT":
            state = GrobMotor.ONLIMIT
        else:
            state = GrobMotor.UNUSABLE

        self.set_is_ready(state > GrobMotor.UNUSABLE)

        if self.motorState != state:
            self.motorState = state
            self.emit("stateChanged", (self.motorState,))

    def get_state(self):
        self.updateState()
        return self.motorState

    def motorLimitsChanged(self):
        self.emit("limitsChanged", (self.get_limits(),))

    def get_limits(self):
        return self.motor.get_limits()

    def positionChanged(self, absolutePosition, private={}):
        if math.fabs(absolutePosition - private.get("old_pos", 1e12)) <= 1e-3:
            return
        private["old_pos"] = absolutePosition

        self.emit("valueChanged", (absolutePosition,))

    def get_value(self):
        return self.motor.read_dial()

    def _set_value(self, value):
        if isinstance(self.motor, self.grob.SampleMotor):
            # position has to be relative
            self.motor.set_value_relative(value - self.get_value())
        else:
            self.motor.start_one(value)

    def waitEndOfMove(self, timeout=None):
        with gevent.Timeout(timeout):
            self.motor.wait_for_move()

    def motorIsMoving(self):
        return self.is_ready() and self.motor.is_moving()

    def get_motor_mnemonic(self):
        return self.motor_name

    def stop(self):
        pass



"""
[Name] BeamInfo

[Description]
BeamInfo hardware object is used to define final beam size and shape.
It can include aperture, slits and/or other beam definer (lenses or other eq.)

[Emited signals]
beamInfoChanged

[Included Hardware Objects]
-----------------------------------------------------------------------
| name            | signals          | functions
-----------------------------------------------------------------------
 aperture_HO	    apertureChanged
 slits_HO
 beam_definer_HO
-----------------------------------------------------------------------
"""

import logging

from mxcubecore import HardwareRepository as HWR
from mxcubecore.BaseHardwareObjects import HardwareObject


class BeamInfo(HardwareObject):
    def __init__(self, *args):
        super().__init__(*args)

        self.beam_size_slits = [9999, 9999]
        self.beam_size_aperture = [9999, 9999]
        self.beam_size_definer = [9999, 9999]
        self.beam_position = [0, 0]

        self.aperture_HO = None
        self.slits_HO = None
        self.beam_info_dict = {}

    def init(self):
        try:
            self.aperture_HO = HWR.get_hardware_repository().get_hardware_object(
                self.get_property("aperture")
            )
            self.connect(self.aperture_HO, "apertureChanged", self.aperture_pos_changed)
        except Exception:
            logging.getLogger("HWR").debug("BeamInfo: aperture not defined correctly")
        try:
            self.slits_HO = HWR.get_hardware_repository().get_hardware_object(
                self.get_property("slits")
            )
            self.connect(self.slits_HO, "gapSizeChanged", self.slits_gap_changed)
        except Exception:
            logging.getLogger("HWR").debug("BeamInfo: slits not defined correctly")
        try:
            self.connect(
                HWR.beamline.beam.beam_definer,
                "definerPosChanged",
                self.definer_pos_changed,
            )
        except Exception:
            logging.getLogger("HWR").debug(
                "BeamInfo: beam definer not defined correctly"
            )

        self.beam_position_hor = self.get_channel_object("beam_position_hor")
        self.beam_position_hor.connect_signal("update", self.beam_pos_hor_changed)
        self.beam_position_ver = self.get_channel_object("beam_position_ver")
        self.beam_position_ver.connect_signal("update", self.beam_pos_ver_changed)
        self.chan_beam_size_microns = self.get_channel_object("beam_size_microns")
        self.chan_beam_shape_ellipse = self.get_channel_object("beam_shape_ellipse")

    def beam_pos_hor_changed(self, value):
        self.beam_position[0] = value
        self.emit("beamPosChanged", (self.beam_position,))

    def beam_pos_ver_changed(self, value):
        self.beam_position[1] = value
        self.emit("beamPosChanged", (self.beam_position,))

    def get_beam_position(self):
        return self.beam_position

    def set_beam_position(self, beam_x, beam_y):
        self.beam_position = [beam_x, beam_y]
        self.beam_position_hor.set_value(int(beam_x))
        self.beam_position_ver.set_value(int(beam_y))

    def aperture_pos_changed(self, nameList, name, size):
        self.beam_size_aperture = size
        self.evaluate_beam_info()
        self.re_emit_values()

    def slits_gap_changed(self, size):
        self.beam_size_slits = size
        self.evaluate_beam_info()
        self.re_emit_values()

    def definer_pos_changed(self, name, size):
        self.beam_size_definer = size
        self.evaluate_beam_info()
        self.re_emit_values()

    def get_beam_info(self):
        return self.evaluate_beam_info()

    def get_beam_size(self):
        """
        Description: returns beam size in microns
        Resturns: list with two integers
        """
        self.evaluate_beam_info()
        return (
            int(self.beam_info_dict["size_x"] * 1000),
            int(self.beam_info_dict["size_y"] * 1000),
        )

    def get_beam_shape(self):
        self.evaluate_beam_info()
        return self.beam_info_dict["shape"]

    def get_slits_gap(self):
        self.evaluate_beam_info()
        return self.beam_size_slits

    def evaluate_beam_info(self):
        """
        Description: called if aperture, slits or focusing has been changed
        Returns: dictionary, {size_x: 0.1, size_y: 0.1, shape: "rectangular"}
        """
        size_x = min(
            self.beam_size_aperture[0],
            self.beam_size_slits[0],
            self.beam_size_definer[0],
        )
        size_y = min(
            self.beam_size_aperture[1],
            self.beam_size_slits[1],
            self.beam_size_definer[1],
        )

        self.beam_info_dict["size_x"] = size_x
        self.beam_info_dict["size_y"] = size_y

        if self.beam_size_aperture < self.beam_size_slits:
            self.beam_info_dict["shape"] = "ellipse"
        else:
            self.beam_info_dict["shape"] = "rectangular"

        return self.beam_info_dict

    def re_emit_values(self):
        if (
            self.beam_info_dict["size_x"] != 9999
            and self.beam_info_dict["size_y"] != 9999
        ):
            self.emit(
                "beamSizeChanged",
                ((self.beam_info_dict["size_x"], self.beam_info_dict["size_y"]),),
            )
            self.emit("beamInfoChanged", (self.beam_info_dict,))
            if self.chan_beam_size_microns is not None:
                self.chan_beam_size_microns.set_value(
                    (
                        self.beam_info_dict["size_x"] * 1000,
                        self.beam_info_dict["size_y"] * 1000,
                    )
                )
            if self.chan_beam_shape_ellipse is not None:
                self.chan_beam_shape_ellipse.set_value(
                    self.beam_info_dict["shape"] == "ellipse"
                )



#
#  Project: MXCuBE
#  https://github.com/mxcube
#
#  This file is part of MXCuBE software.
#
#  MXCuBE is free software: you can redistribute it and/or modify
#  it under the terms of the GNU Lesser General Public License as published by
#  the Free Software Foundation, either version 3 of the License, or
#  (at your option) any later version.
#
#  MXCuBE is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#  GNU Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public License
#  along with MXCuBE. If not, see <http://www.gnu.org/licenses/>.

"""
EMBLPiezoMotor
"""

import logging

from mxcubecore.HardwareObjects.EMBL.TINEMotor import TINEMotor

__credits__ = ["EMBL Hamburg"]
__license__ = "LGPLv3+"
__category__ = "Motor"


class EMBLPiezoMotor(TINEMotor):
    """
    Based on the TineMotor. After the move command executes additional
    commands.
    """

    def __init__(self, name):
        TINEMotor.__init__(self, name)

        self.cmd_move_first = None
        self.cmd_move_second = None

    def init(self):
        TINEMotor.init(self)
        self.cmd_move_first = self.get_command_object("cmdMoveFirst")
        self.cmd_move_second = self.get_command_object("cmdMoveSecond")

    def _set_value(self, value):
        """Moves motor to the target position

        :param value: target position
        :type value: float
        :return: None
        """
        self.cmd_move_first(value)
        self.cmd_move_second(1)
